package ru.dbotthepony.kstarbound

import org.apache.logging.log4j.LogManager
import org.lwjgl.Version
import ru.dbotthepony.kstarbound.client.StarboundClient
import ru.dbotthepony.kstarbound.defs.TileDefinition
import ru.dbotthepony.kstarbound.math.Vector2d
import ru.dbotthepony.kstarbound.world.Chunk
import ru.dbotthepony.kstarbound.world.ChunkPos
import ru.dbotthepony.kstarbound.world.entities.Humanoid
import java.io.File

private val LOGGER = LogManager.getLogger()

fun main() {
	LOGGER.info("Running LWJGL ${Version.getVersion()}")

	val client = StarboundClient()

	Starbound.addFilePath(File("./unpacked_assets/"))

	Starbound.initializeGame { finished, replaceStatus, status ->
		client.pushLoadingText(status, replaceStatus)
	}

	client.onTermination {
		Starbound.terminateLoading = true
	}

	var chunkA: Chunk? = null

	Starbound.onInitialize {
		chunkA = client.world!!.computeIfAbsent(ChunkPos(0, 0)).chunk
		val chunkB = client.world!!.computeIfAbsent(ChunkPos(-1, 0)).chunk
		val chunkC = client.world!!.computeIfAbsent(ChunkPos(-2, 0)).chunk

		val tile = Starbound.getTileDefinition("alienrock")

		for (x in -48 .. 48) {
			for (y in 0 .. 20) {
				val chnk = client.world!!.computeIfAbsent(ChunkPos(x, y))

				for (bx in 0 .. 31) {
					for (by in 0 .. 3) {
						chnk.chunk.foreground[bx, by] = tile
					}
				}
			}
		}

		for (x in 0 .. 31) {
			for (y in 0 .. 3) {
				chunkA!!.foreground[x, y] = tile
				chunkC.foreground[x, y] = tile
			}
		}

		for (x in 0 .. 31) {
			for (y in 8 .. 9) {
				chunkA!!.foreground[x, y] = tile
				chunkC.foreground[x, y] = tile
			}
		}

		for (x in 0 .. 31) {
			for (y in 0 .. 0) {
				chunkB.foreground[x, y] = tile
			}
		}

		for (x in 4 .. 8) {
			for (y in 4 .. 8) {
				chunkA!!.foreground[x, y] = null as TileDefinition?
			}
		}

		/*val rand = Random()

		for (i in 0 .. 400) {
			chunkA!!.foreground[rand.nextInt(0, CHUNK_SIZE_FF), rand.nextInt(0, CHUNK_SIZE_FF)] = tile
		}*/
	}

	//val rand = Random()
	val ent = Humanoid(client.world!!)

	ent.pos += Vector2d(y = 36.0, x = 10.0)

	client.onDrawGUI {
		client.gl.font.render("${ent.pos}", y = 100f, scale = 0.25f)
	}

	client.onPostDrawWorld {
		client.gl.quadWireframe {
			it.quad(ent.aabb + ent.pos)
		}
	}

	while (client.renderFrame()) {
		Starbound.pollCallbacks()

		ent.moveAndCollide(client.frameRenderTime)
		client.camera.pos.x = ent.pos.x.toFloat()
		client.camera.pos.y = ent.pos.y.toFloat()

		//println(client.camera.velocity.toDoubleVector() * client.frameRenderTime * 0.1)

		//if (ent.onGround)
			ent.velocity += client.camera.velocity.toDoubleVector() * client.frameRenderTime * 0.1

		//if (chunkA != null && glfwGetTime() < 10.0) {
		//	val tile = Starbound.getTileDefinition("alienrock")
			//chunkA!!.foreground[rand.nextInt(0, CHUNK_SIZE_FF), rand.nextInt(0, CHUNK_SIZE_FF)] = tile
		//}
	}
}
