package ru.dbotthepony.kstarbound.test

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import ru.dbotthepony.kstarbound.math.AABB
import ru.dbotthepony.kstarbound.math.Vector2d
import ru.dbotthepony.kstarbound.math.roundByAbsoluteValue

object MathTests {
	@Test
	@DisplayName("roundByAbsoluteValue test")
	fun roundByAbsoluteValueTest() {
		check(roundByAbsoluteValue(0.0) == 0)
		check(roundByAbsoluteValue(0.1) == 1)
		check(roundByAbsoluteValue(1.1) == 2)
		check(roundByAbsoluteValue(-0.1) == -1)
		check(roundByAbsoluteValue(-0.0) == 0)
		check(roundByAbsoluteValue(-1.0) == -1)
		check(roundByAbsoluteValue(-1.1) == -2)
	}

	@Test
	@DisplayName("AABB Basic Math")
	fun basicAABB() {
		val a = AABB.rectangle(Vector2d.ZERO, 1.0, 1.0)

		check(a.intersect(AABB.rectangle(Vector2d(-1.0), 1.0, 1.0)))
		check(!a.intersectWeak(AABB.rectangle(Vector2d(-1.0), 1.0, 1.0)))
		check(!a.intersect(AABB.rectangle(Vector2d(-2.0), 1.0, 1.0)))
		check(!a.intersectWeak(AABB.rectangle(Vector2d(-2.0), 1.0, 1.0)))

		check(a.intersect(AABB.rectangle(Vector2d(-0.9), 1.0, 1.0)))
		check(a.intersectWeak(AABB.rectangle(Vector2d(-0.9), 1.0, 1.0)))

		val bigA = AABB.rectangle(Vector2d.ZERO, 200.0, 200.0)
		val smallB = AABB.rectangle(Vector2d.ZERO, 1.0, 1.0)

		check(bigA.intersect(smallB))
		check(smallB.intersect(bigA))

		check(bigA.intersectWeak(smallB))
		check(smallB.intersectWeak(bigA))

		check(AABB.rectangle(Vector2d.ZERO, 1.0, 1.0) == AABB(Vector2d(-0.5, -0.5), Vector2d(0.5, 0.5)))

		val combineA = AABB(Vector2d(0.0, 0.0), Vector2d(2.0, 2.0))
		val combineB = AABB(Vector2d(2.0, 5.0), Vector2d(4.0, 6.0))

		check(combineA.combine(combineB) == AABB(Vector2d(0.0, 0.0), Vector2d(4.0, 6.0))) { combineA.combine(combineB).toString() }
	}
}
